@@ -16,6 +16,8 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
@@ -25,12 +27,11 @@
 #include <string.h>
 #include <signal.h>
 #include <locale.h>
+#include <unistd.h>
 
 #include <gtk/gtk.h>
 #include <gmodule.h>
 
-#include "config.h"
-
 #if (defined (HAVE_LIBPOPT) && defined (HAVE_POPT_H)) || defined (GNOME)
 #define HAVE_POPT
 #endif
@@ -56,10 +57,14 @@
 #include "layer_dialog.h"
 #include "load_save.h"
 #include "preferences.h"
-#include "custom.h"
+#include "dia_dirs.h"
 #include "render_eps.h"
 #include "render_svg.h"
 
+#if defined(HAVE_LIBPNG) && defined(HAVE_LIBART)
+extern DiaExportFilter png_export_filter;
+#endif
+
 static void register_all_objects(void);
 static void register_all_sheets(void);
 static int name_is_lib(char *name);
@@ -178,6 +183,8 @@ app_init (int argc, char **argv)
   dia_image_init();
 #endif
 
+  gdk_rgb_init();
+
   gtk_rc_parse ("diagtkrc"); 
 
   /*  enable_core_dumps(); */
@@ -202,6 +209,9 @@ app_init (int argc, char **argv)
   filter_register_export(&dia_export_filter);
   filter_register_export(&eps_export_filter);
   filter_register_export(&svg_export_filter);
+#if defined(HAVE_LIBPNG) && defined(HAVE_LIBART)
+  filter_register_export(&png_export_filter);
+#endif
 
   debug_break();
 
@@ -275,26 +285,40 @@ app_exit(void)
   
   if (diagram_modified_exists()) {
     GtkWidget *dialog;
+    GtkWidget *vbox;
     GtkWidget *label;
     GtkWidget *button;
     int result = FALSE;
 
+#ifdef GNOME
+    dialog = gnome_dialog_new(_("Quit, are you sure?"), NULL);
+    vbox = GNOME_DIALOG(dialog)->vbox;
+    gnome_dialog_set_close(GNOME_DIALOG(dialog), TRUE);
+#else
     dialog = gtk_dialog_new();
-  
+    vbox = GTK_DIALOG(dialog)->vbox;
+    gtk_window_set_title (GTK_WINDOW (dialog), _("Quit, are you sure?"));
+    gtk_container_set_border_width (GTK_CONTAINER (dialog), 0);
+#endif
+
     gtk_signal_connect (GTK_OBJECT (dialog), "destroy", 
 			GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
     
-    gtk_window_set_title (GTK_WINDOW (dialog), _("Quit, are you sure?"));
-    gtk_container_set_border_width (GTK_CONTAINER (dialog), 0);
     label = gtk_label_new (_("Modified diagrams exists.\n"
 			   "Are you sure you want to quit?"));
   
     gtk_misc_set_padding (GTK_MISC (label), 10, 10);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), 
-			label, TRUE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
   
     gtk_widget_show (label);
 
+#ifdef GNOME
+    gnome_dialog_append_button_with_pixmap(GNOME_DIALOG(dialog),
+					   _("Quit"), GNOME_STOCK_PIXMAP_QUIT);
+    gnome_dialog_append_button(GNOME_DIALOG(dialog),GNOME_STOCK_BUTTON_CANCEL);
+
+    result = (gnome_dialog_run(GNOME_DIALOG(dialog)) == 0);
+#else
     button = gtk_button_new_with_label (_("Quit"));
     GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
     gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), 
@@ -325,6 +349,7 @@ app_exit(void)
     gtk_grab_add(dialog);
 
     gtk_main();
+#endif
 
     if (result == FALSE)
       return;
@@ -490,8 +515,6 @@ register_all_objects(void)
   
   object_register_type(&group_type);
 
-  custom_register_objects();
-
   home_path = getenv("HOME");
   library_path = getenv("DIA_LIB_PATH");
   if (library_path != NULL)
@@ -511,7 +534,9 @@ register_all_objects(void)
     }
     g_free(library_path);
   } else {
-    register_objects_in(LIBDIR "/dia");
+    char *thedir = dia_get_lib_directory("dia");
+    register_objects_in(thedir);
+    g_free(thedir);
   }
 
 }
