@@ -52,242 +52,16 @@
 #include "connectionpoint_ops.h"
 #include "undo.h"
 #include "pagesetup.h"
+#include "dia_dirs.h"
 
 GdkImlibImage *logo;
 
-/* if user already has a diagram open, then start out in that directory */
-static void set_default_file_selection_directory (GtkWidget *fs)
-{
-  DDisplay *ddisp = ddisplay_active();
-  if (ddisp && ddisp->diagram && (! ddisp->diagram->unsaved)) {
-    char *last_slash;
-    char *fn = (char *) malloc (strlen (ddisp->diagram->filename) + 10);
-    strcpy (fn, ddisp->diagram->filename);
-    last_slash = strrchr (fn, '/');
-    if (last_slash) {
-      *(last_slash+1) = '\0';
-      gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs), fn);
-    }
-    free (fn);
-  }
-}
-
-
-/* if the a diagram is open and has a file name, set a default
-   file name is given extension */
-static void set_default_file_selection_extension (GtkWidget *fs, char *ext)
-{
-  DDisplay *ddisp = ddisplay_active();
-  if (ddisp && ddisp->diagram && (! ddisp->diagram->unsaved)) {
-    char *last_slash;
-    char *last_dot;
-    char *ext_index;
-    char *fn = (char *) malloc (strlen (ddisp->diagram->filename) +
-				10 + strlen (ext));
-    strcpy (fn, ddisp->diagram->filename);
-    
-    /* put a .ext extention on the file name */
-    last_slash = strrchr (fn, '/');
-    last_dot = strrchr (fn, '.');
-    if (last_slash && last_dot && (last_dot > last_slash))
-      ext_index = last_dot;
-    else if ((! last_slash) && last_dot)
-      ext_index = last_dot;
-    else
-      ext_index = fn + strlen (fn);
-    strcpy (ext_index, ext);
-    
-    gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs), fn);
-    free (fn);
-  }
-}
-
 
 void file_quit_callback(GtkWidget *widget, gpointer data)
 {
   app_exit();
 }
 
-static void
-file_open_dialog_ok_callback (GtkWidget        *w,
-			      GtkFileSelection *fs)
-{
-  char *filename;
-  Diagram *diagram;
-  DDisplay *ddisp;
-  
-  filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
-  diagram = diagram_load(filename);
-
-  if (diagram != NULL) {
-    diagram_update_extents(diagram);
-    layer_dialog_set_diagram(diagram);
-  
-    ddisp = new_display(diagram);
-  }
-  /* Error messages are done in diagram_load() */
-
-  gtk_widget_destroy (GTK_WIDGET (fs));
-}
-
-void file_open_callback(GtkWidget *widget, gpointer data)
-{
-  GtkWidget *window = NULL;
-
-  window = gtk_file_selection_new (_("Open diagram"));
-
-  gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
-
-  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (window)->ok_button),
-		      "clicked", GTK_SIGNAL_FUNC(file_open_dialog_ok_callback),
-		      window);
-  
-  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (window)->cancel_button),
-			     "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
-			     GTK_OBJECT (window));
-
-  set_default_file_selection_directory (window);
-  
-  gtk_widget_show (window);
-}
-
-void
-file_save_callback(GtkWidget *widget, gpointer data)
-{
-  DDisplay *ddisp;
-
-  ddisp = ddisplay_active();
-
-  if (ddisp->diagram->unsaved) {
-    file_save_as_callback(widget, data);
-  } else {
-    diagram_update_extents(ddisp->diagram);
-    diagram_save(ddisp->diagram, ddisp->diagram->filename);
-  }
-}
-
-static void
-set_true_callback(GtkWidget *w, int *data)
-{
-  *data = TRUE;
-}
-
-static void
-file_save_as_dialog_ok_callback (GtkWidget        *w,
-				 GtkFileSelection *fs)
-{
-  char *filename;
-  Diagram *dia;
-  struct stat stat_struct;
-  
-  dia = gtk_object_get_user_data(GTK_OBJECT(fs));
-  
-  filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
-
-  if (stat(filename, &stat_struct)==0) {
-    GtkWidget *dialog = NULL;
-    GtkWidget *button, *label;
-    char buffer[300];
-    int result;
-
-    dialog = gtk_dialog_new();
-  
-    gtk_signal_connect (GTK_OBJECT (dialog), "destroy", 
-			GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
-    
-    gtk_window_set_title (GTK_WINDOW (dialog), _("File already exists"));
-    gtk_container_set_border_width (GTK_CONTAINER (dialog), 0);
-    g_snprintf(buffer, 300,
-	       _("The file '%s' already exists.\n"
-		 "Do you want to overwrite it?"), filename);
-    label = gtk_label_new (buffer);
-  
-    gtk_misc_set_padding (GTK_MISC (label), 10, 10);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), 
-			label, TRUE, TRUE, 0);
-  
-    gtk_widget_show (label);
-
-    result = FALSE;
-    
-    button = gtk_button_new_with_label (_("Yes"));
-    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), 
-			button, TRUE, TRUE, 0);
-    gtk_widget_grab_default (button);
-    gtk_signal_connect (GTK_OBJECT (button), "clicked",
-			GTK_SIGNAL_FUNC(set_true_callback),
-			&result);
-    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			       GTK_OBJECT (dialog));
-    gtk_widget_show (button);
-    
-    button = gtk_button_new_with_label (_("No"));
-    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area),
-			button, TRUE, TRUE, 0);
-    
-    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			       GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			       GTK_OBJECT (dialog));
-    
-    gtk_widget_show (button);
-
-    gtk_widget_show (dialog);
-
-    /* Make dialog modal: */
-    gtk_widget_grab_focus(dialog);
-    gtk_grab_add(dialog);
-
-    gtk_main();
-
-    if (result==FALSE) {
-      gtk_grab_remove(GTK_WIDGET(fs));
-      gtk_widget_destroy(GTK_WIDGET(fs));
-      return;
-    }
-  }
-
-  diagram_update_extents(dia);
-
-  diagram_set_filename(dia, filename);
-  diagram_save(dia, filename);
-  
-  gtk_grab_remove(GTK_WIDGET(fs));
-  gtk_widget_destroy(GTK_WIDGET(fs));
-}
-
-void
-file_save_as_callback(GtkWidget *widget, gpointer data)
-{
-  DDisplay *ddisp;
-  GtkWidget *window = NULL;
-
-  ddisp = ddisplay_active();
-
-  window = gtk_file_selection_new (_("Save diagram"));
-  gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
-  
-  gtk_object_set_user_data(GTK_OBJECT(window), ddisp->diagram);
-  
-  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (window)->ok_button),
-		      "clicked", GTK_SIGNAL_FUNC(file_save_as_dialog_ok_callback),
-		      window);
-  
-  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (window)->cancel_button),
-			     "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
-			     GTK_OBJECT (window));
-
-  set_default_file_selection_directory (window);
-  
-  gtk_widget_show (window);
-
-  /* Make dialog modal: */
-  gtk_widget_grab_focus(window);
-  gtk_grab_add(window);
-}
-
 static void
 file_import_from_xfig_dialog_ok_callback (GtkWidget        *w,
 				     GtkFileSelection *fs)
@@ -544,20 +318,22 @@ help_about_callback(GtkWidget *widget, gpointer data)
   gtk_container_add (GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), vbox);
 
   if (!logo) {
-      g_snprintf(str, sizeof(str), "%s/dia_logo.png", DATADIR);
+      gchar* datadir = dia_get_data_directory(""); 
+      g_snprintf(str, sizeof(str), "%s/dia_logo.png", datadir);
       logo = gdk_imlib_load_image(str);
-      gdk_imlib_render(logo, logo->rgb_width, logo->rgb_height);
+      g_free(datadir);
   }
 
   if (logo) {
-    frame = gtk_frame_new (NULL);
-    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
-    gtk_container_set_border_width (GTK_CONTAINER (frame), 1);
-    gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, TRUE, 1);
+      gdk_imlib_render(logo, logo->rgb_width, logo->rgb_height);
+      frame = gtk_frame_new (NULL);
+      gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
+      gtk_container_set_border_width (GTK_CONTAINER (frame), 1);
+      gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, TRUE, 1);
 
-    gpixmap = gtk_pixmap_new(logo->pixmap, logo->shape_mask);
+      gpixmap = gtk_pixmap_new(logo->pixmap, logo->shape_mask);
 
-    gtk_container_add (GTK_CONTAINER(frame), gpixmap);
+      gtk_container_add (GTK_CONTAINER(frame), gpixmap);
   }
 
   frame = gtk_frame_new (NULL);
@@ -677,6 +453,31 @@ view_show_cx_pts_callback(gpointer callback_data,
   }
 }
 
+#ifdef GNOME
+void
+view_aa_callback(GtkWidget *widget,
+		 gpointer callback_data)
+#else /* GNOME */
+void
+view_aa_callback(gpointer callback_data,
+		 guint callback_action,
+		 GtkWidget *widget)
+#endif /* GNOME */
+{
+  DDisplay *ddisp;
+  int aa;
+
+  ddisp = ddisplay_active();
+  
+  aa = GTK_CHECK_MENU_ITEM (widget)->active;
+
+  if (aa != ddisp->aa_renderer) {
+    ddisplay_set_renderer(ddisp, aa);
+    ddisplay_add_update_all(ddisp);
+    ddisplay_flush(ddisp);
+  }
+}
+
 #ifdef GNOME
 void
 view_visible_grid_callback(GtkWidget *widget,
@@ -793,8 +594,8 @@ view_show_all_callback(GtkWidget *widget, gpointer data)
   ddisp = ddisplay_active();
   dia = ddisp->diagram;
 
-  width = ddisp->renderer->renderer.pixel_width;
-  height = ddisp->renderer->renderer.pixel_height;
+  width = ddisp->renderer->pixel_width;
+  height = ddisp->renderer->pixel_height;
   
   magnify_x = (real)width /
     (dia->data->extents.right - dia->data->extents.left) / ddisp->zoom_factor;
