@@ -20,6 +20,10 @@
 #include <math.h>
 
 #include "config.h"
+#ifdef GNOME
+#include <gnome.h>
+#endif
+
 #include "intl.h"
 
 #include "display.h"
@@ -110,6 +114,7 @@ new_display(Diagram *dia)
 
   ddisp->autoscroll = TRUE;
 
+  ddisp->aa_renderer = 0;
   ddisp->renderer = NULL;
   
   ddisp->update_areas = NULL;
@@ -157,14 +162,28 @@ display_hash(DDisplay *ddisp)
   return (gulong) ddisp;
 }
 
+void
+ddisplay_transform_coords_double(DDisplay *ddisp,
+				 coord x, coord y,
+				 double *xi, double *yi)
+{
+  Rectangle *visible = &ddisp->visible;
+  double width = ddisp->renderer->pixel_width;
+  double height = ddisp->renderer->pixel_height;
+  
+  *xi = (x - visible->left)  * (real)width / (visible->right - visible->left);
+  *yi = (y - visible->top)  * (real)height / (visible->bottom - visible->top);
+}
+
+
 void
 ddisplay_transform_coords(DDisplay *ddisp,
 			  coord x, coord y,
 			  int *xi, int *yi)
 {
   Rectangle *visible = &ddisp->visible;
-  int width = ddisp->renderer->renderer.pixel_width;
-  int height = ddisp->renderer->renderer.pixel_height;
+  int width = ddisp->renderer->pixel_width;
+  int height = ddisp->renderer->pixel_height;
   
   *xi = ROUND ( (x - visible->left)  * (real)width /
 		(visible->right - visible->left) );
@@ -193,8 +212,8 @@ ddisplay_untransform_coords(DDisplay *ddisp,
 			    coord *x, coord *y)
 {
   Rectangle *visible = &ddisp->visible;
-  int width = ddisp->renderer->renderer.pixel_width;
-  int height = ddisp->renderer->renderer.pixel_height;
+  int width = ddisp->renderer->pixel_width;
+  int height = ddisp->renderer->pixel_height;
   
   *x = visible->left + xi*(visible->right - visible->left) / (real)width;
   *y = visible->top +  yi*(visible->bottom - visible->top) / (real)height;
@@ -231,8 +250,8 @@ ddisplay_add_update(DDisplay *ddisp, Rectangle *rect)
   Rectangle *r;
   int top,bottom,left,right;
   Rectangle *visible;
-  int width = ddisp->renderer->renderer.pixel_width;
-  int height = ddisp->renderer->renderer.pixel_height;
+  int width = ddisp->renderer->pixel_width;
+  int height = ddisp->renderer->pixel_height;
 
   if (!rectangle_intersects(rect, &ddisp->visible))
     return;
@@ -300,7 +319,7 @@ ddisplay_flush(DDisplay *ddisp)
   
   /* Renders updates to pixmap + copies display_areas to canvas(screen) */
 
-  renderer = &ddisp->renderer->renderer;
+  renderer = ddisp->renderer;
 
   (renderer->interactive_ops->clip_region_clear)(renderer);
 
@@ -339,9 +358,14 @@ ddisplay_flush(DDisplay *ddisp)
   while(l!=NULL) {
     ir = (IRectangle *) l->data;
 
-    renderer_gdk_copy_to_window(ddisp->renderer, ddisp->canvas->window,
-				ir->left, ir->top,
-				ir->right - ir->left, ir->bottom - ir->top);
+    if (ddisp->aa_renderer)
+      renderer_libart_copy_to_window((RendererLibart *)ddisp->renderer, ddisp->canvas->window,
+				     ir->left, ir->top,
+				     ir->right - ir->left, ir->bottom - ir->top);
+    else 
+      renderer_gdk_copy_to_window((RendererGdk *)ddisp->renderer, ddisp->canvas->window,
+				  ir->left, ir->top,
+				  ir->right - ir->left, ir->bottom - ir->top);
     
     l = g_slist_next(l);
   }
@@ -385,13 +409,13 @@ ddisplay_render_pixmap(DDisplay *ddisp, Rectangle *update)
     return;
   }
 
-  renderer = &ddisp->renderer->renderer;
+  renderer = ddisp->renderer;
 
   /* Erase background */
   (renderer->interactive_ops->fill_pixel_rect)(renderer,
 					       0, 0,
-					       renderer->pixel_width,
-					       renderer->pixel_height,
+					       renderer->pixel_width-1,
+					       renderer->pixel_height-1,
 					       &ddisp->diagram->data->bg_color);
 
   /* Draw grid */
@@ -453,8 +477,8 @@ ddisplay_set_origo(DDisplay *ddisp, coord x, coord y)
 {
   Rectangle *extents = &ddisp->diagram->data->extents;
   Rectangle *visible = &ddisp->visible;
-  int width = ddisp->renderer->renderer.pixel_width;
-  int height = ddisp->renderer->renderer.pixel_height;
+  int width = ddisp->renderer->pixel_width;
+  int height = ddisp->renderer->pixel_height;
     
   /*  updaterar origo+visible+rulers */
   ddisp->origo.x = x;
@@ -631,15 +655,44 @@ void ddisplay_scroll_right(DDisplay *ddisp)
   ddisplay_scroll(ddisp, &delta);
 }
 
+void
+ddisplay_set_renderer(DDisplay *ddisp, int aa_renderer)
+{
+  int width, height;
+  if (ddisp->aa_renderer)
+    destroy_libart_renderer((RendererLibart *)ddisp->renderer);
+  else
+    destroy_gdk_renderer((RendererGdk *)ddisp->renderer);
+
+  ddisp->aa_renderer = aa_renderer;
+
+  width = ddisp->canvas->allocation.width;
+  height = ddisp->canvas->allocation.height;
+  
+  if (ddisp->aa_renderer){
+    ddisp->renderer = (Renderer *)new_libart_renderer(ddisp);
+    libart_renderer_set_size((RendererLibart *)ddisp->renderer, ddisp->canvas->window, width, height);
+  } else {
+    ddisp->renderer = (Renderer *)new_gdk_renderer(ddisp);
+    gdk_renderer_set_size((RendererGdk *)ddisp->renderer, ddisp->canvas->window, width, height);
+  }
+}
+
 void
 ddisplay_resize_canvas(DDisplay *ddisp,
 		       int width,  int height)
 {
   if (ddisp->renderer==NULL) {
-    ddisp->renderer = new_gdk_renderer(ddisp);
+    if (ddisp->aa_renderer)
+      ddisp->renderer = (Renderer *)new_libart_renderer(ddisp);
+    else
+      ddisp->renderer = (Renderer *)new_gdk_renderer(ddisp);
   }
   
-  gdk_renderer_set_size(ddisp->renderer, ddisp->canvas->window, width, height);
+  if (ddisp->aa_renderer)
+    libart_renderer_set_size((RendererLibart *)ddisp->renderer, ddisp->canvas->window, width, height);
+  else
+    gdk_renderer_set_size((RendererGdk *)ddisp->renderer, ddisp->canvas->window, width, height);
 
   ddisplay_set_origo(ddisp, ddisp->origo.x, ddisp->origo.y);
 
@@ -690,8 +743,6 @@ extern DDisplay *ddisplay_active(void)
 static void
 are_you_sure_close_dialog_no(GtkWidget *widget, GtkWidget *dialog)
 {
-  gtk_grab_remove(dialog);
-  gtk_widget_hide(dialog);
   gtk_widget_destroy(dialog);
 }
 
@@ -703,8 +754,6 @@ are_you_sure_close_dialog_yes(GtkWidget *widget,
 
   ddisp =  gtk_object_get_user_data(GTK_OBJECT(dialog));
   
-  gtk_grab_remove(dialog);
-  gtk_widget_hide(dialog);
   gtk_widget_destroy(dialog);
   gtk_widget_destroy (ddisp->shell);
 }
@@ -714,7 +763,7 @@ void
 ddisplay_close(DDisplay *ddisp)
 {
   Diagram *dia;
-  GtkWidget *dialog;
+  GtkWidget *dialog, *vbox;
   GtkWidget *label;
   GtkWidget *button;
   
@@ -725,27 +774,42 @@ ddisplay_close(DDisplay *ddisp)
     gtk_widget_destroy (ddisp->shell);
     return;
   }
-  
-  dialog = gtk_dialog_new();
 
+#ifdef GNOME
+  dialog = gnome_dialog_new(_("Really close?"),
+			    GNOME_STOCK_BUTTON_YES,GNOME_STOCK_BUTTON_NO,NULL);
+  gnome_dialog_set_default(GNOME_DIALOG(dialog), 0);
+  vbox = GNOME_DIALOG(dialog)->vbox;
+#else
+  dialog = gtk_dialog_new();
   gtk_window_set_title (GTK_WINDOW (dialog), _("Really close?"));
   gtk_container_set_border_width (GTK_CONTAINER (dialog), 0);
+  vbox = GTK_DIALOG(dialog)->vbox;
+#endif
   
   label = gtk_label_new (_("This diagram has not been saved.\n"
 			 "Are you sure you want to close this window?"));
   
   gtk_misc_set_padding (GTK_MISC (label), 10, 10);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), 
-		      label, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
   
   gtk_widget_show (label);
-  
+
+  gtk_object_set_user_data(GTK_OBJECT(dialog), ddisp);
+
+#ifdef GNOME
+  gnome_dialog_button_connect(GNOME_DIALOG(dialog), 0,
+			      GTK_SIGNAL_FUNC(are_you_sure_close_dialog_yes),
+			      dialog);
+  gnome_dialog_button_connect(GNOME_DIALOG(dialog), 1,
+			      GTK_SIGNAL_FUNC(are_you_sure_close_dialog_no),
+			      dialog);
+#else
   button = gtk_button_new_with_label (_("Yes"));
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), 
 		      button, TRUE, TRUE, 0);
   gtk_widget_grab_default (button);
-  gtk_object_set_user_data(GTK_OBJECT(dialog), ddisp);
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
 		      GTK_SIGNAL_FUNC(are_you_sure_close_dialog_yes),
 		      dialog);
@@ -761,10 +825,10 @@ ddisplay_close(DDisplay *ddisp)
 		      dialog);
 
   gtk_widget_show (button);
+#endif
 
   /* Make dialog modal: */
-  gtk_widget_grab_focus(dialog);
-  gtk_grab_add(dialog);
+  gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
   
   gtk_widget_show(dialog);
 }
@@ -823,8 +887,12 @@ ddisplay_really_destroy(DDisplay *ddisp)
   diagram_remove_ddisplay(dia, ddisp);
 
   grid_destroy_dialog(&ddisp->grid);
-  
-  destroy_gdk_renderer(ddisp->renderer);
+
+  if (ddisp->aa_renderer)
+    destroy_libart_renderer((RendererLibart *)ddisp->renderer);
+  else
+    destroy_gdk_renderer((RendererGdk *)ddisp->renderer);
+    
   
   g_hash_table_remove(display_ht, ddisp->shell);
   g_hash_table_remove(display_ht, ddisp->canvas);
