@@ -102,6 +102,10 @@ static GnomeUIInfo viewmenu[] = {
   GNOMEUIINFO_ITEM_NONE(N_("Zoom _Out"), N_("Zoom out 50%"), view_zoom_out_callback),
   GNOMEUIINFO_SUBTREE(N_("_Zoom"), zoommenu),
   GNOMEUIINFO_ITEM_NONE(N_("Edit Grid..."), NULL, view_edit_grid_callback),
+#ifdef HAVE_LIBART  
+  GNOMEUIINFO_TOGGLEITEM(N_("_AntiAliased"), NULL,
+			 view_aa_callback, NULL),
+#endif  
   GNOMEUIINFO_TOGGLEITEM(N_("_Visible Grid"), NULL,
 			 view_visible_grid_callback, NULL),
   GNOMEUIINFO_TOGGLEITEM(N_("_Snap To Grid"), NULL,
@@ -181,7 +185,7 @@ static GtkItemFactoryEntry toolbox_menu_items[] =
   {N_("/File/_Preferences..."),NULL,         file_preferences_callback, 0 },
   {N_("/File/sep1"),           NULL,         NULL,                      0, "<Separator>"},
   {N_("/File/_Quit"),          "<control>Q", file_quit_callback,        0 },
-  {N_("/_Help"),               NULL,         NULL,                      0, "<LastBranch>" },
+  {N_("/_Help"),               NULL,         NULL,                      0, "<Branch>" },
   {N_("/Help/_About"),         NULL,         help_about_callback,       0 },
 };
 
@@ -224,6 +228,9 @@ static GtkItemFactoryEntry display_menu_items[] =
   {N_("/View/Zoom/35.4%"),        NULL,         view_zoom_set_callback,   354},
   {N_("/View/Zoom/25%"),          NULL,         view_zoom_set_callback,   250},
   {N_("/View/Edit Grid..."),      NULL,         view_edit_grid_callback,    0},
+#ifdef HAVE_LIBART  
+  {N_("/View/_AntiAliased"),      NULL,         view_aa_callback,           0, "<CheckItem>"},
+#endif
   {N_("/View/_Visible Grid"),     NULL,         view_visible_grid_callback, 0, "<CheckItem>"},
   {N_("/View/_Snap To Grid"),     "G",          view_snap_to_grid_callback, 0, "<CheckItem>"},
   {N_("/View/Show _Rulers"),      NULL,         view_toggle_rulers_callback,0, "<CheckItem>"},
