@@ -1,4 +1,4 @@
-/* xxxxxx -- an diagram creation/manipulation program
+/* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,6 +27,7 @@ struct _ScrollTool {
   Tool tool;
 
   int scrolling;
+  int use_hand;
   Point last_pos;
 };
 
