@@ -16,8 +16,14 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include <glib.h>
 #include <math.h>
 
+#ifdef G_OS_WIN32
+#include <float.h>
+#define finite(d) _finite(d)
+#endif
+
 #include "arrows.h"
 
 static void
@@ -30,7 +36,7 @@ calculate_arrow(Point *poly/*[3]*/, Point *to, Point *from,
   
   delta = *to;
   point_sub(&delta, from);
-  len = sqrt(point_dot(&delta, &delta));
+  len = point_len(&delta);
   if (len <= 0.0001) {
     delta.x=1.0;
     delta.y=0.0;
@@ -71,6 +77,59 @@ draw_lines(Renderer *renderer, Point *to, Point *from,
       renderer->ops->draw_polyline(renderer, poly, 3, color);
 }
 
+static void 
+draw_fill_ellipse(Renderer *renderer, Point *to, Point *from,
+	     real length, real width, real linewidth,
+	     Color *fg_color,Color *bg_color)
+{
+  BezPoint bp[5];
+  Point vl,vt;
+
+  renderer->ops->set_linewidth(renderer, linewidth);
+  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
+  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+
+  if (!bg_color) {
+    /* no bg_color means filled ellipse ; we then compensate for the line width
+     */
+    length += linewidth;
+    width += linewidth;
+  }
+  point_copy(&vl,from); point_sub(&vl,to);
+  point_normalize(&vl);
+  if (!finite(vl.x)) {
+    vl.x = 1.0; vl.y = 0.0;
+  }
+  point_get_perp(&vt,&vl);
+
+
+  /* This pile of crap is quite well handled by gcc. */ 
+  bp[0].type = BEZ_MOVE_TO;
+  point_copy(&bp[0].p1,to);
+  bp[1].type = bp[2].type = bp[3].type = bp[4].type = BEZ_CURVE_TO;
+  point_copy(&bp[4].p3,&bp[0].p1);
+
+  point_copy_add_scaled(&bp[2].p3,&bp[0].p1,&vl,length);
+  point_copy_add_scaled(&bp[2].p2,&bp[2].p3,&vt,-width / 4.0);
+  point_copy_add_scaled(&bp[3].p1,&bp[2].p3,&vt,width / 4.0);
+  point_copy_add_scaled(&bp[1].p1,&bp[0].p1,&vt,-width / 4.0);
+  point_copy_add_scaled(&bp[4].p2,&bp[0].p1,&vt,width / 4.0);
+  point_copy_add_scaled(&bp[1].p3,&bp[0].p1,&vl,length / 2.0); /* temp */
+  point_copy_add_scaled(&bp[3].p3,&bp[1].p3,&vt,width / 2.0);
+  point_add_scaled(&bp[1].p3,&vt,-width / 2.0);
+  point_copy_add_scaled(&bp[1].p2,&bp[1].p3,&vl,-length / 4.0);
+  point_copy_add_scaled(&bp[4].p1,&bp[3].p3,&vl,-length / 4.0);
+  point_copy_add_scaled(&bp[2].p1,&bp[1].p3,&vl,length / 4.0);
+  point_copy_add_scaled(&bp[3].p2,&bp[3].p3,&vl,length / 4.0);
+  if (bg_color) {
+    renderer->ops->fill_bezier(renderer,bp,sizeof(bp)/sizeof(bp[0]),bg_color);
+    renderer->ops->draw_bezier(renderer,bp,sizeof(bp)/sizeof(bp[0]),fg_color);
+  } else {
+    renderer->ops->fill_bezier(renderer,bp,sizeof(bp)/sizeof(bp[0]),fg_color);
+  }
+}
+
 /* Only draw the upper part of the arrow */
 static void
 draw_halfhead(Renderer *renderer, Point *to, Point *from,
@@ -193,6 +252,64 @@ fill_diamond(Renderer *renderer, Point *to, Point *from,
   renderer->ops->fill_polygon(renderer, poly, 4, color);
 }
 
+static void
+calculate_slashed_cross(Point *poly/*[6]*/, Point *to, Point *from,
+			real length, real width)
+{
+  Point delta;
+  Point orth_delta;
+  real len;
+  int i;
+  
+  delta = *to;
+  point_sub(&delta, from);
+  len = sqrt(point_dot(&delta, &delta));
+  if (len <= 0.0001) {
+    delta.x=1.0;
+    delta.y=0.0;
+  } else {
+    delta.x/=len;
+    delta.y/=len;
+  }
+  
+  orth_delta.x = delta.y;
+  orth_delta.y = -delta.x;
+
+  point_scale(&delta, length/2.0);
+  point_scale(&orth_delta, width/2.0);
+
+  for(i=0; i<6;i++)poly[i] = *to;
+
+  point_add(&poly[1], &delta);
+
+  point_add(&poly[2], &delta);
+  point_add(&poly[2], &orth_delta);
+
+  point_sub(&poly[3], &delta);
+  point_sub(&poly[3], &orth_delta);
+
+  point_add(&poly[4], &orth_delta);
+  point_sub(&poly[5], &orth_delta);
+}
+
+static void
+draw_slashed_cross(Renderer *renderer, Point *to, Point *from,
+     real length, real width, real linewidth, Color *color)
+{
+  Point poly[6];
+  
+  calculate_slashed_cross(poly, to, from, length, width);
+  
+  renderer->ops->set_linewidth(renderer, linewidth);
+  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
+  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  
+  renderer->ops->draw_line(renderer, &poly[0],&poly[1], color);
+  renderer->ops->draw_line(renderer, &poly[2],&poly[3], color);                   
+  renderer->ops->draw_line(renderer, &poly[4],&poly[5], color);
+}
+
 void
 arrow_draw(Renderer *renderer, ArrowType type,
 	   Point *to, Point *from,
@@ -222,7 +339,21 @@ arrow_draw(Renderer *renderer, ArrowType type,
   case ARROW_FILLED_DIAMOND:
     fill_diamond(renderer, to, from, length, width, fg_color);
     break;
+  case ARROW_SLASHED_CROSS:
+    draw_slashed_cross(renderer, to, from, length, width, linewidth, fg_color);
+    break;
+  case ARROW_FILLED_ELLIPSE:
+    draw_fill_ellipse(renderer,to,from,length,width,linewidth,
+		      fg_color,NULL);
+    break;
+  case ARROW_HOLLOW_ELLIPSE:
+    draw_fill_ellipse(renderer,to,from,length,width,linewidth,
+		      fg_color,bg_color);
+    break;
   }
   
 }
 
+
+
+
