@@ -603,6 +603,8 @@ dependency_get_properties(Dependency *dep)
     dep->properties_dialog = prop_dialog;
 
     dialog = gtk_vbox_new(FALSE, 0);
+    gtk_object_ref(GTK_OBJECT(dialog));
+    gtk_object_sink(GTK_OBJECT(dialog));
     prop_dialog->dialog = dialog;
     
     hbox = gtk_hbox_new(FALSE, 5);
