@@ -235,6 +235,8 @@ ellipse_get_properties(Ellipse *ellipse)
     ellipse_properties_dialog = g_new(EllipsePropertiesDialog, 1);
 
     vbox = gtk_vbox_new(FALSE, 5);
+    gtk_object_ref(GTK_OBJECT(vbox));
+    gtk_object_sink(GTK_OBJECT(vbox));
     ellipse_properties_dialog->vbox = vbox;
 
     hbox = gtk_hbox_new(FALSE, 5);
